ul.jd_menu, 
ul.jd_menu_vertical {
	margin: 0px;
	padding: 0px;
	list-style-type: none;
}
ul.jd_menu ul,
ul.jd_menu_vertical ul {
	display: none;
}
ul.jd_menu li {
	float: left;
}
ul.jd_menu_vertical {
	width: 300px;
}
ul.jd_menu_vertical li {
	float: none;
}

/* -- Sub-Menus -- */
ul.jd_menu ul,
ul.jd_menu_vertical ul {
	position: absolute;
	display: none;
	list-style-type: none;
	margin: 0px;
	padding: 0px;
	z-index: 10000;
}
ul.jd_menu ul li,
ul.jd_menu_vertical ul li {
	float: none;
	margin: 0px;
}


ul.jd_menu,
ul.jd_menu ul,
ul.jd_menu_vertical,
ul.jd_menu_vertical ul {
	background-color: #369;
	border: 1px solid #036;
	border-top: 1px solid #69C;
	border-left: 1px solid #69C;
	
	height: 20px;
}
ul.jd_menu_vertical {
	height: auto;
}
ul.jd_menu ul {
	height: auto;
}
* html ul.jd_menu ul {
	width: 1%;
}
ul.jd_menu li {
	font-family: sans-serif;
	font-size: 12px;
	color: #FFF;
	
	line-height: 14px;
	
	margin: 0px;
	padding: 4px 7px 3px 7px;
	height: 13px;
	
	cursor: pointer;
	white-space: nowrap;
}
ul.jd_menu li li {
	width: 200px;
}
ul.jd_menu li a {
	color: #FFF;
	text-decoration: none;
}

ul.jd_menu li.jdm_hover,
ul.jd_menu li.jdm_active {
	background-color: #69C;
	color: #FFF;
	
	padding: 3px 6px 2px 6px;
	border: 1px solid #369;
	border-left: 1px solid #9CF;
	border-top: 1px solid #9CF;
	
}
ul.jd_menu li.right{
	float: right;
}
